from tokenizer import Token


class Evaluator:
    def __init__(self, tokenes: list[str]):
        self.tokenes = tokenes
        self.memory = [0 for n in range(100000)]
        self.p = 0
        self.now_token_position = 0
        self.corresponding_params = []

    def eval(self):
        self.set_corresponding_params()
        while True:

            if self.now_token_position >= len(self.tokenes):
                break
            t = self.tokenes[self.now_token_position]
            if t == Token.PLUS:
                self.memory[self.p] += 1
                self.now_token_position +=1
                continue
            if t == Token.MINUS:
                self.memory[self.p] -= 1
                self.now_token_position +=1
                continue
            if t == Token.LEFT:
                self.p -= 1
                self.now_token_position += 1
                continue
            if t == Token.RIGHT:
                self.p += 1
                self.now_token_position += 1
                continue
            if t == Token.LEFT_PARAM:

                if self.memory[self.p] == 0:
                    for cp in self.corresponding_params:
                        if cp[0] == self.now_token_position:
                            self.now_token_position = cp[1]
                            break
                else:
                    self.now_token_position+=1
                continue
            if t == Token.RIGHT_PARAM:
                if self.memory[self.p] == 0:
                    self.now_token_position+=1
                else:
                    for cp in self.corresponding_params:
                        if cp[1] == self.now_token_position:
                            self.now_token_position = cp[0]
                            break
                continue
            if t == Token.DOT:
                print(chr(self.memory[self.p]), end="")
                self.now_token_position += 1
                continue




    def set_corresponding_params(self):
        for i, t in enumerate(self.tokenes):
            if t == Token.LEFT_PARAM:
                self.get_position_corresponding_param(i)




    def get_position_corresponding_param(self, pos):
        left_param_pos = pos
        depth = 0
        while pos:
            if pos >= len(self.tokenes) :
                return -1
            t = self.tokenes[pos]
            if t == Token.LEFT_PARAM:
                depth += 1
            if t == Token.RIGHT_PARAM:
                depth -= 1
                if depth == 0:
                    self.corresponding_params.append((left_param_pos, pos))
                    return pos
            pos += 1



from chkw.evaluator import Evaluator
from chkw.tokenizer import Tokenizer
from chkw.transcompiler import TransCompiler


import os
import click


@click.group(invoke_without_command=True)
@click.pass_context
@click.argument('filename', required=False)
@click.option('-trans', 'trans', is_flag=True, help='Process the file with the "trans" action')
def cli(ctx, filename, trans):

    if ctx.invoked_subcommand is None:

        if trans and filename:
            process_trans(filename)
        elif filename:
            process(filename)
        else:
            click.echo('No command specified')

def process(filename):
    with open(filename, "r") as f:
        code = f.read()
    tokenes = Tokenizer(code).tokenize()
    Evaluator(tokenes).eval()

def process_trans(filename):
    click.echo(f'Trans compiling {filename} ...')
    with open(filename, "r") as f:
        code = TransCompiler(f.read()).compile()
    base = os.path.basename(filename)
    name, _ = os.path.splitext(base)
    with open(name + ".chkw", "w") as f:
        f.write(code)

    click.echo(f'Compiled file {name + ".chkw"} is saved')





if __name__ == '__main__':
    cli()



# code = "+++++++++[>++++++++>+++++++++++>+++>+<<<<-]>.>++.+++++++..+++.>+++++.<<+++++++++++++++.>.+++.------.--------.>+.>+."
#
#
# print(TransCompiler(code).compile())
# code = TransCompiler(code).compile()
# tokenes = Tokenizer(code).tokenize()
# Evaluator(tokenes).eval()









from transcompiler import TransCompiler


class Token:
    PLUS = "+"
    MINUS = "-"
    DOT = "."
    KAMMA = ","
    LEFT = "<"
    RIGHT = ">"
    LEFT_PARAM = "["
    RIGHT_PARAM = "]"


class Tokenizer:

    def __init__(self, code :str):
        self.tc_map = TransCompiler(code).map
        self.code = code
        self.map = {
            self.tc_map["+"] : Token.PLUS,
            self.tc_map["."] : Token.DOT,
            self.tc_map["<"] : Token.LEFT,
            self.tc_map[">"] : Token.RIGHT,
            self.tc_map["["] : Token.LEFT_PARAM,
            self.tc_map["]"] : Token.RIGHT_PARAM,
            self.tc_map[","] : Token.KAMMA,
            self.tc_map["-"] : Token.MINUS,
        }


    def tokenize(self) -> list[str]:
        tokenes : list[str] = []
        tmp = ""
        for c in self.code:
            tmp += c
            if tmp in self.tc_map.values():
                tokenes.append(self.map[tmp])
                tmp = ""
        return tokenes


class TransCompiler:
    def __init__(self, code :str):
        self.code = code
        self.map = {
            "+": "ヤハッ",
            "-": "アァ...",
            ".": "ッテコト..?",
            "<" : "ウワァ..." ,
            ">" : "ヒャハー...!",
            "[" : "ウゥ...",
            "]" : "泣いちゃった..!",
            "," :"なんだもう朝かと",
        }

    def compile(self):
        result = ""
        for c in self.code:
            result += self.map[c]

        return result
